"""Usage examples of the MyRIO class: digital inputs and outputs

This examples show how to use the DIO ports and the
onboard button and LEDs.

We have a set of myRIOs in our facilities that we complement
with some MXP cards. These cards have an RGB LED connected to
DIO_2:0 and two push buttons on DIO_4:3. We use these channels
in our examples. The port that we use most is the A port, so
we set it as default on our package.

Last update: 2024/03/07 Aitzol Ezeiza Ramos (UPV/EHU)
"""

import myrio_base
import time

myrio1 = myrio_base.MyRIO()

# Read digital port (the whole port), one channel, and the onboard button

print("Digital port A:")
print(myrio1.read_digital_port(port="A"))
print("Digital channel 3, port A:")
print(myrio1.read_digital_input(3, port="A"))
print("onboard button:")
print(myrio1.read_button())

# Write onboard LEDs, using an integer and using a list of Booleans

print("Switching on the onboard LEDS (15 in binary)")
myrio1.write_leds_integer(7)
time.sleep(2)
print("Switching off the onboard LEDS (0 in binary)")
myrio1.write_leds_integer(0)
time.sleep(2)
print("Switching on the onboard LEDS 1 and 4 ([True, False, False, True])")
myrio1.write_leds_booleans([True, False, False, True])
time.sleep(2)
print("Switching off all the onboard LEDS ([False, False, False, False])")
myrio1.write_leds_booleans([False, False, False, False])
time.sleep(2)

# myRIO digital output test (the whole port at once, and then one by one)

print("Switching on the three RGB LEDS (7 in binary)")
myrio1.write_digital_port(7)
time.sleep(2)
print("Switching off the three RGB LEDS and all the other outputs (0,0 in binary)")
myrio1.write_digital_port(0, 0)
time.sleep(2)

print("Switching on the green LED (channel 0)")
myrio1.write_digital_output(0, True)
time.sleep(1)
print("Switching on the red LED (channel 1)")
myrio1.write_digital_output(1, True)
time.sleep(1)
print("Switching on the blue LED (channel 2)")
myrio1.write_digital_output(2, True)
time.sleep(1)

print("Switching off the three RGB LEDS one by one")
myrio1.write_digital_output(0, False)
time.sleep(1)
myrio1.write_digital_output(1, False)
time.sleep(1)
myrio1.write_digital_output(2, False)
time.sleep(1)

print("One color at a time (3 reps)")
for i in range(3):
    myrio1.write_digital_port(myrio_base.RED)
    time.sleep(1)
    myrio1.write_digital_port(myrio_base.GREEN)
    time.sleep(1)
    myrio1.write_digital_port(myrio_base.BLUE)
    time.sleep(1)

print("End of digital tests")
myrio1.write_digital_port(myrio_base.RGB_OFF)



import myrio_api_client as myAPIc
from time import sleep

myRIO = myAPIc.MyRIO_API_Client()
# myRIO = MyRIO_API_Client(ip_address='172.22.11.3',port:8080)

print("Accelerometer:")
print(myRIO.get_onboard_accelerometer())
print("Onboard button:")
print(myRIO.get_onboard_button())
print("Digital Input 3:")
print(myRIO.get_digital_input(3))
print("Digital Input 0 (port B):")
print(myRIO.get_digital_input(3, port_in="B"))
print("Analog Input 1:")
print(myRIO.get_analog_input(1))
print("Onboard LEDS (all ON)")
myRIO.set_onboard_leds(15)
sleep(1)
print("Onboard LEDS (all OFF)")
myRIO.set_onboard_leds(0)
sleep(1)
print("Digital Output 0 ON")
myRIO.set_digital_output(0, 1)
sleep(1)
print("Digital Output 0 OFF")
myRIO.set_digital_output(0, 0)
sleep(1)
print("Analog Output 0 5.0V")
myRIO.set_analog_output(0, 5.0)
print("Analog Input 2: (the same if wired)")
print(myRIO.get_analog_input(2))
sleep(1)
print("Analog Output 0 0.0V")
myRIO.set_analog_output(0, 0.0)
print("Analog Input 2: (the same if wired)")
print(myRIO.get_analog_input(2))
print("MXP temperature")
print(myRIO.get_mxp_temperature())
print("MXP luminosity")
print(myRIO.get_mxp_luminosity())
print("MXP buttons")
print(myRIO.get_mxp_button(1))
print(myRIO.get_mxp_button(2))
print("MXP RGB LED set GREEN")
myRIO.set_mxp_rgb_color(1)
sleep(1)
print("MXP RGB LED set RED")
myRIO.set_mxp_rgb_color(2)
sleep(1)
print("MXP RGB LED set BLUE")
myRIO.set_mxp_rgb_color(4)
sleep(1)
print("MXP RGB LED set WHITE")
myRIO.set_mxp_rgb_color(7)
sleep(1)
print("MXP RGB LED set OFF")
myRIO.set_mxp_rgb_color(0)
sleep(1)
print("PWM RGB LED set purple")
myRIO.set_pwm_output(0, 50.0, port_in="B")
myRIO.set_pwm_output(1, 0.0, port_in="B")
myRIO.set_pwm_output(2, 50.0, port_in="B")
sleep(1)
print("PWM RGB LED paint it black")
myRIO.set_pwm_output(0, 0.0, port_in="B")
myRIO.set_pwm_output(1, 0.0, port_in="B")
myRIO.set_pwm_output(2, 0.0, port_in="B")



"""Usage examples of the MyRIO class: analog inputs and outputs

This examples show how to use the analog inputs and outputs
and the onboard Accelerometer.

We have a set of myRIOs in our facilities that we complement
with some MXP cards. These cards have a temperature sensor in
AI0 and a Light sensor in AI1. We use these channels
in our examples. The port that we use most is the A port, so
we set it as default on our package.

Last update: 2024/03/07 Aitzol Ezeiza Ramos (UPV/EHU)
"""

import myrio_base
import time

myrio1 = myrio_base.MyRIO()

# Read onboard accelerometer values
print("Accelerometer test (5 seconds)")
for i in range(10):
    print(myrio1.read_analog_accelerometer())
    time.sleep(0.5)
print("End of Accelerometer test")
time.sleep(2)

# Read analog inputs
print("Analog Inputs test (5 seconds) (values in volts)")
for i in range(10):
    print(str(myrio1.read_analog_input(0)) + " " + str(myrio1.read_analog_input(1)))
    time.sleep(0.5)
print("End of Analog Inputs test")
time.sleep(2)

# Test the analog output wiring it to AI2
print("myRIO analog output test - wire AO0 to AI2 and press the onboard button")

while not myrio1.read_button():
    time.sleep(0.1)

print("AO0 write: 5.0")
myrio1.write_analog_output(channel=0, value=5.0)
time.sleep(1)
print("AI2 read: ", str(myrio1.read_analog_input(channel=2)))
print("AO0 write: 2.5")
myrio1.write_analog_output(channel=0, value=2.5)
time.sleep(1)
print("AI2 read: ", str(myrio1.read_analog_input(channel=2)))
print("AO0 write: 0.0")
myrio1.write_analog_output(channel=0, value=0.0)
time.sleep(1)
print("AI2 read: ", str(myrio1.read_analog_input(channel=2)))
print("End of Analog Output test")



"""myRIO API: A RESTful API server for the myRIO

Last update: 2024/03/14 Aitzol Ezeiza Ramos UPV/EHU

This library uses Flask and Waitress to create an API
based on myRIO_library. Not all the functions are
served this way, but the basics are there:
    · Digital Inputs and Outputs
    · Analog Inputs and Outputs
    · onboard button and LEDs
    · onboard accelerometer
    - MXP board components
The default port is 8080. Some examples of API calls
(using curl) would be the following:

curl -X POST http://172.22.11.2:8080/digital_output/2/1

This turns on the digital output DIO2 on the default MXP port (A)
If you use Windows Powershell, you should replace the -X with -method X

curl -method POST http://172.22.11.2:8080/digital_output/2/1

You can change the default port with a parameter:

curl -X GET http://172.22.11.2:8080/digital_input/3?port=B

Read the examples un the examples folder for more info.
"""

from flask import Flask, jsonify, request
import myrio_base

DEFAULT_HTTP_PORT = 8080

app = Flask(__name__)


@app.before_first_request
def setup():
    """We need to create a session before any API call
    We use a global handler for that.
    """
    global myrio_handler
    myrio_handler = myRIO_base.MyRIO()


@app.route("/digital_input/<int:channel_in>", methods=["GET"])
def get_digital_input(channel_in: int):
    """Returns the value (true/false) of a digital input"""
    port_in = request.args.get("port", default="A", type=str)
    value = myrio_handler.read_digital_input(channel=channel_in, port=port_in)
    return jsonify({"value": value})


@app.route("/digital_output/<int:channel_in>/<int:value_in>", methods=["POST"])
def set_digital_output(channel_in: int, value_in: int):
    """Sets the value (true 1, false 0) of a digital output"""
    port_in = request.args.get("port", default="A", type=str)
    myrio_handler.write_digital_output(
        channel=channel_in, port=port_in, value=bool(value_in)
    )
    return jsonify({"success": True})


@app.route("/analog_input/<int:channel_in>", methods=["GET"])
def get_analog_input(channel_in: int):
    """Returns the value (volts in float type) of an analog input"""
    port_in = request.args.get("port", default="A", type=str)
    value = myrio_handler.read_analog_input(channel=channel_in, port=port_in)
    return jsonify({"value": value})


@app.route("/analog_output/<int:channel_in>/<float:value_in>", methods=["POST"])
def set_analog_output(channel_in: int, value_in: float):
    """Sets the value (volts in float type) of an analog output"""
    port_in = request.args.get("port", default="A", type=str)
    myrio_handler.write_analog_output(channel=channel_in, port=port_in, value=value_in)
    return jsonify({"success": True})


# Define routes for the onboard inputs and outputs
@app.route("/onboard_button", methods=["GET"])
def get_onboard_button():
    """Returns the value (true/false) of the onboard button"""
    value = myrio_handler.read_button()
    return jsonify({"value": value})


@app.route("/onboard_leds/<int:value_in>", methods=["POST"])
def set_onboard_leds(value_in: int):
    """Sets the value (0..15 integer) of the onboard LEDs"""
    myrio_handler.write_leds_integer(value_in)
    return jsonify({"success": True})


@app.route("/onboard_accelerometer", methods=["GET"])
def get_onboard_accelerometer():
    """Returns the value (x, y, z floats) of the onboard accelerometer"""
    values = myrio_handler.read_analog_accelerometer()
    response_data = {"x": values[0], "y": values[1], "z": values[2]}
    return jsonify(response_data)


# MXP board components: buttons, RGB LED, Temperature sensor, Light sensor


@app.route("/mxp_button/<int:button_in>", methods=["GET"])
def get_mxp_button(button_in: int):
    """Returns the value (true/false) of one of the buttons.
    We expect 1 or 2, first (black) and second (white).
    """
    port_in = request.args.get("port", default="A", type=str)
    value = myrio_handler.read_MXP_button(button=button_in, port=port_in)
    return jsonify({"value": value})


@app.route("/mxp_rgb_color/<int:color_in>", methods=["POST"])
def set_mxp_rgb_color(color_in: int):
    """Sets the color (0 to 7) of the MXP RGB LED
    Remember that the order is G(0)R(1)B(2)
    Green is 1, Red is 2, Blue is 4
    White is 7 and Off is 0.
    """
    port_in = request.args.get("port", default="A", type=str)
    myrio_handler.write_MXP_RGB_LED(color=color_in, port=port_in)
    return jsonify({"success": True})


@app.route("/mxp_temperature", methods=["GET"])
def get_mxp_temperature():
    """Returns the temperature (degrees in float type) of
    the NTC temperature sensor (MXP board)
    """
    port_in = request.args.get("port", default="A", type=str)
    value = myrio_handler.read_MXP_temperature(port=port_in)
    return jsonify({"value": value})


@app.route("/mxp_luminosity", methods=["GET"])
def get_mxp_luminosity():
    """Returns the luminosity (percentage in float type) of
    the LDR light sensor (MXP board)
    """
    port_in = request.args.get("port", default="A", type=str)
    value = myrio_handler.read_MXP_luminosity(port=port_in)
    return jsonify({"value": value})


# Extra functions: set PWM output


@app.route("/pwm_output/<int:channel_in>/<float:value_in>", methods=["POST"])
def set_pwm_output(channel_in: int, value_in: float):
    """Sets the value (duty cycle in float type) of a PWM output"""
    port_in = request.args.get("port", default="A", type=str)
    max_value = myrio_handler.config_PWM_output(channel=channel_in, port=port_in)
    myrio_handler.write_PWM_output(
        channel=channel_in, port=port_in, duty_cycle=value_in, X=max_value
    )
    return jsonify({"success": True})


if __name__ == "__main__":
    from waitress import serve

    serve(app, host="0.0.0.0", port=DEFAULT_HTTP_PORT)
    # 0.0.0.0 means it will be served in all network interfaces

    # Instead of serving, you can debug the application
    # using Flask's server
    # app.run(debug=True)



"""Usage examples of the MyRIO class: play waveform

This example shows how to play a waveform using the MyRIO class.

Playing wav files is not supported in the myRIO library. However,
there is a rudimentary way to play a waveform using raw data.

Last update: 2024/04/10 Aitzol Ezeiza Ramos (UPV/EHU)
"""

import myrio_base as myRIO
import pkg_resources

myrio1 = myRIO.MyRIO()

# Play a simple waveform using raw data
print("Playing a simple waveform")
csv_file = pkg_resources.resource_filename("myRIO_base", "examples/PacManDeath.csv")
print("Waveform file: ", csv_file)
my_waveform = myRIO.extract_waveform_from_csv_file(csv_file)
myrio1.play_waveform(my_waveform)



"""myRIO API client: An API client for the myRIO API server

Last update: 2024/03/15 Aitzol Ezeiza Ramos UPV/EHU

This is the client of the myRIO API server. Please refer to the
server documentation for further help.
The basics are:
    · Digital Inputs and Outputs
    · Analog Inputs and Outputs
    · onboard button and LEDs
    · onboard accelerometer
    - mxp board components
The default port is 8080. Some examples of API calls
(using curl) would be the following:

curl -X POST http://172.22.11.2:8080/digital_output/2/1

This turns on the digital output DIO2 on the default MXP port (A)
If you use Windows Powershell, you should replace the -X with -method X

curl -method POST http://172.22.11.2:8080/digital_output/2/1

You can change the default port with a parameter:

curl -X GET http://172.22.11.2:8080/digital_input/3?port=B

Read the examples un the examples folder for more info.
"""

import requests
from typing import Tuple

DEFAULT_HTTP_PORT = 8080
DEFAULT_HOST_IP = "172.22.11.2"


class MyRIO_API_Client:
    """class MyRIO_API_Client
    This is the base class for making requests to the API.
    We define generic methods for further development and
    developer flexibility. Anyway, there will be specific
    methods for each application route of the API server.

    Developer methods:
        make_request
        get_data
        post_data
        put_data
        delete_data

    Basic methods:
        get_digital_input
        set_digital_output
        get_analog_input
        set_analog_output
        get_onboard_button
        set_onboard_leds
        get_onboard_accelerometer

    MXP board methods:
        get_mxp_button
        set_mxp_rgb_color
        get_mxp_temperature
        get_mxp_luminosity

    Extra methods:
        set_pwm_output
    """

    def __init__(
        self, ip_address: str = DEFAULT_HOST_IP, port: int = DEFAULT_HTTP_PORT
    ):
        self.base_url = "http://" + ip_address + ":" + str(DEFAULT_HTTP_PORT)

    def make_request(self, method, endpoint, data=None, params=None):
        url = self.base_url + "/" + endpoint
        response = requests.request(method, url, json=data, params=params)
        response.raise_for_status()
        json_response = response.json()

        # Check if the response is a success message
        if isinstance(json_response, dict) and json_response.get("success") is True:
            return None  # Success message, no need to process
        # Check for a single key-value pair
        if isinstance(json_response, dict) and len(json_response) == 1:
            key, value = next(iter(json_response.items()))
            return value
        # Check for a dict of three key-value pairs (accelerometer)
        if isinstance(json_response, dict) and len(json_response) == 3:
            return list(json_response.values())
        # If none of the above cases match, return the full JSON response
        else:
            return json_response

    def get_data(self, endpoint, params=None):
        return self.make_request("GET", endpoint, params=params)

    def post_data(self, endpoint, data=None):
        return self.make_request("POST", endpoint, data=data)

    def put_data(self, endpoint, data=None):
        return self.make_request("PUT", endpoint, data=data)

    def delete_data(self, endpoint, params=None):
        return self.make_request("DELETE", endpoint, params=params)

    def get_digital_input(self, channel_in: int, port_in: str = "A") -> bool:
        """Returns the value (true/false) of a digital input"""
        if port_in == "A":
            endpoint = "digital_input/" + str(channel_in)
        else:
            endpoint = "digital_input/" + str(channel_in) + "?port=" + port_in

        response = self.get_data(endpoint)
        return response

    def set_digital_output(self, channel_in: int, value_in: int, port_in: str = "A"):
        """Sets the value (true 1, false 0) of a digital output"""
        if port_in == "A":
            endpoint = "digital_output/" + str(channel_in) + "/" + str(value_in)
        else:
            endpoint = (
                "digital_output/"
                + str(channel_in)
                + "/"
                + str(value_in)
                + "?port="
                + port_in
            )
        self.post_data(endpoint)

    def get_analog_input(self, channel_in: int, port_in: str = "A") -> float:
        """Returns the value (volts in float type) of an analog input"""
        if port_in == "A":
            endpoint = "analog_input/" + str(channel_in)
        else:
            endpoint = "analog_input/" + str(channel_in) + "?port=" + port_in
        response = self.get_data(endpoint)
        return float(response)

    def set_analog_output(self, channel_in: int, value_in: float, port_in: str = "A"):
        """Sets the value (volts in float type) of an analog output"""
        if port_in == "A":
            endpoint = "analog_output/" + str(channel_in) + "/" + str(value_in)
        else:
            endpoint = (
                "analog_output/"
                + str(channel_in)
                + "/"
                + str(value_in)
                + "?port="
                + port_in
            )
        self.post_data(endpoint)

    def get_onboard_button(self) -> bool:
        """Returns the value (true/false) of the onboard button"""
        endpoint = "onboard_button"
        response = self.get_data(endpoint)
        return response

    def set_onboard_leds(self, value_in: int):
        """Sets the value (0..15 integer) of the onboard LEDs"""
        endpoint = "onboard_leds/" + str(value_in)
        response = self.post_data(endpoint)

    def get_onboard_accelerometer(self) -> Tuple[float, float, float]:
        """Returns the value (x, y, z floats) of the onboard accelerometer"""
        endpoint = "onboard_accelerometer"
        response = self.get_data(endpoint)
        return response

    """
    MXP board methods:
        get_mxp_button
        set_mxp_rgb_color
        get_mxp_temperature
        get_mxp_luminosity
    """

    def get_mxp_button(self, button_in: int, port_in: str = "A") -> bool:
        """Returns the value (true/false) of one of the MXP buttons"""
        if port_in == "A":
            endpoint = "mxp_button/" + str(button_in)
        else:
            endpoint = "mxp_button/" + str(button_in) + "?port=" + port_in

        response = self.get_data(endpoint)
        return response

    def set_mxp_rgb_color(self, color_in: int):
        """Sets the value (0..7 integer) of the MXP RGB LED"""
        endpoint = "mxp_rgb_color/" + str(color_in)
        response = self.post_data(endpoint)

    def get_mxp_temperature(self, port_in: str = "A") -> float:
        """Returns the temperature (degrees in float type)
        of the MXP NTC temperature sensor (AI0)
        """
        if port_in == "A":
            endpoint = "mxp_temperature"
        else:
            endpoint = "mxp_temperature" + "?port=" + port_in
        response = self.get_data(endpoint)
        return float(response)

    def get_mxp_luminosity(self, port_in: str = "A") -> float:
        """Returns the luminosity (percentage in float type)
        of the MXP LDR light sensor (AI1)
        """
        if port_in == "A":
            endpoint = "mxp_luminosity"
        else:
            endpoint = "mxp_luminosity" + "?port=" + port_in
        response = self.get_data(endpoint)
        return float(response)

    # Extra methods

    def set_pwm_output(self, channel_in: int, value_in: float, port_in: str = "A"):
        """Sets the value (duty cycle in float type) of a PWM output"""
        if port_in == "A":
            endpoint = "pwm_output/" + str(channel_in) + "/" + str(value_in)
        else:
            endpoint = (
                "pwm_output/"
                + str(channel_in)
                + "/"
                + str(value_in)
                + "?port="
                + port_in
            )
        self.post_data(endpoint)


if __name__ == "__main__":
    myRIO = MyRIO_API_Client()
    print("Accelerometer:")
    print(myRIO.get_onboard_accelerometer())
    print("MXP temperature")
    print(myRIO.get_mxp_temperature())
    print("More examples in ./examples/client_examples.py")



from setuptools import setup, find_packages

# read the contents of your README file
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="myrio_library",
    version="1.4.5",
    author="Aitzol Ezeiza Ramos",
    author_email="aitzol.ezeiza@ehu.eus",
    description="A library to control the myRIO board from National Instruments",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    package_data={
        "myrio_base": ["data/*"],
        "myrio_base": ["examples/*"],
        "myrio_api": ["examples/*"],
        "myrio_api_client": ["examples/*"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Other OS",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.5",
)
